﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Storage;
using Windows.ApplicationModel.DataTransfer;
using Windows.UI.Xaml.Media.Imaging;
using Windows.Storage.Streams;

namespace MyApp
{
    /// <summary>
    /// 可用于自身或导航至 Frame 内部的空白页。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        private async void OnDragenter(object sender, DragEventArgs e)
        {
            // 获取Deferral是必须，稍后要用它来向系统报告操作完成
            var deferral = e.GetDeferral();
            DataPackageView dataview = e.DataView;
            // 验证数据类型，如果不是文件，就没戏了
            if (dataview.Contains(StandardDataFormats.StorageItems))
            {
                // 取出被拖进来的文件列表
                // 因为用户可能觉得好玩
                // 一次性拖一大堆文件或目录进来
                var items = await dataview.GetStorageItemsAsync();
                if (items.Count > 0)
                {
                    IStorageItem item = items[0];
                    // 这里只关心文件，如果拖的是目录，那就不玩了
                    if (item.IsOfType(StorageItemTypes.File))
                    {
                        // 设置一个有效的拖放操作，只要不是None就行
                        // 其他值都无所谓，主要区别是拖放时的光标显示不同
                        // 但后面我会把光标隐藏，以免影响视线
                        e.AcceptedOperation = DataPackageOperation.Link;
                        StorageFile file = (StorageFile)item;
                        // 得到文件预览图
                        var t = await file.GetScaledImageAsThumbnailAsync(Windows.Storage.FileProperties.ThumbnailMode.SingleItem, 150);
                        BitmapImage bmp = new BitmapImage();
                        bmp.DecodePixelWidth = 150;
                        bmp.SetSource(t);
                        // 设置拖动过程中显示的图标
                        e.DragUIOverride.SetContentFromBitmapImage(bmp);
                        // Caption属性表示在拖动时显示的提示文本
                        // 想看效果的话，就把下面这行代码取消注释
                        //e.DragUIOverride.Caption = file.Name;
                        // 不显示提示文本
                        // 所以，如果你想看上面的Caption值，请把
                        // IsCaptionVisible也改为true
                        e.DragUIOverride.IsCaptionVisible = false;
                        // 干脆连那个指针符号也隐藏了吧
                        e.DragUIOverride.IsGlyphVisible = false;
                    }
                    else
                    {
                        // 不是文件数据，就设置为None，无操作
                        e.AcceptedOperation = DataPackageOperation.None;
                    }
                }
            }
            else
            {
                e.AcceptedOperation = DataPackageOperation.None;
            }
            VisualStateManager.GoToState(this, "DragIn", false);
            // 告诉系统，我干完活了
            deferral.Complete();
        }

        private void OnDragleave(object sender, DragEventArgs e)
        {
            var d = e.GetDeferral();
            VisualStateManager.GoToState(this, "Generic", false);
            d.Complete();
        }

        private async void OnDrop(object sender, DragEventArgs e)
        {
            // 记得获取Deferral对象
            var def = e.GetDeferral();
            DataPackageView data = e.DataView;
            // 还是再验证一下吧，防止意外
            if (data.Contains(StandardDataFormats.StorageItems))
            {
                var storageItems = await data.GetStorageItemsAsync();
                if (storageItems.Count > 0)
                {
                    IStorageItem item = storageItems[0];
                    if (item.IsOfType(StorageItemTypes.File))
                    {
                        StorageFile file = item as StorageFile;
                        // 生成内存图像
                        using (var inStream = await file.OpenReadAsync())
                        {
                            BitmapImage bmp = new BitmapImage();
                            bmp.DecodePixelWidth = 300;
                            bmp.SetSource(inStream);
                            img.Source = bmp; //显示图像
                        }
                    }
                }
            }
            VisualStateManager.GoToState(this, "Generic", false);
            // 报告操作系统，处理完成
            def.Complete();
        }
    }
}
